package apgas.util;

import static apgas.Constructs.here;
import static apgas.Constructs.place;

import apgas.Configuration;
import java.io.Serializable;
import java.time.ZonedDateTime;
import java.time.format.DateTimeFormatter;

public class ConsolePrinter implements Serializable {

  private static final boolean PRINT = Configuration.APGAS_CONSOLEPRINTER.get();

  private static final ConsolePrinter instance = new ConsolePrinter();

  private static final long startTime = System.nanoTime();

  public static synchronized ConsolePrinter getInstance() {
    return ConsolePrinter.instance;
  }

  public synchronized void printlnWithoutAPGAS(String output) {
    if (PRINT == true) {
      String callerName = Thread.currentThread().getStackTrace()[2].getMethodName();
      long time = (long) ((System.nanoTime() - startTime) / 1E6);
      System.out.println("(in " + callerName + ") [" + time + "]: " + output);
    }
  }

  public synchronized void println(String output) {
    //    if (PRINT == true) {
    //      String callerName = Thread.currentThread().getStackTrace()[2].getMethodName();
    //      long time = (long) ((System.nanoTime() - startTime) / 1E6);
    //      System.out.println(here() + " (in " + callerName + ") [" + time + "]: " + output);
    //    }
    if (PRINT == true) {
      printlnAlways(output);
    }
  }

  public synchronized void printlnErr(String output) {
    if (PRINT == true) {
      String callerName = Thread.currentThread().getStackTrace()[2].getMethodName();
      long time = (long) ((System.nanoTime() - startTime) / 1E6);
      System.err.println(here() + " (in " + callerName + ") [" + time + "]: " + output);
    }
  }

  public synchronized void print(String output) {
    if (PRINT == true) {
      String callerName = Thread.currentThread().getStackTrace()[2].getMethodName();
      long time = (long) ((System.nanoTime() - startTime) / 1E6);
      System.out.print(here() + " (in " + callerName + ") [" + time + "]: " + output);
    }
  }

  public synchronized void printErr(String output) {
    if (PRINT == true) {
      String callerName = Thread.currentThread().getStackTrace()[2].getMethodName();
      long time = (long) ((System.nanoTime() - startTime) / 1E6);
      System.err.print(here() + " (in " + callerName + ") [" + time + "]: " + output);
    }
  }

  public synchronized void remotePrintln(int source, String output) {
    if (PRINT == true) {
      String callerName = Thread.currentThread().getStackTrace()[2].getMethodName();
      long time = (long) ((System.nanoTime() - startTime) / 1E6);
      System.out.println(
          place(source)
              + " (in "
              + callerName
              + " at "
              + here().id
              + ") ["
              + time
              + "]: "
              + output);
    }
  }

  public synchronized void remotePrintlnErr(int source, String output) {
    if (PRINT == true) {
      String callerName = Thread.currentThread().getStackTrace()[2].getMethodName();
      long time = (long) ((System.nanoTime() - startTime) / 1E6);
      System.err.println(
          place(source)
              + " (in "
              + callerName
              + " at "
              + here().id
              + ") ["
              + time
              + "]: "
              + output);
    }
  }

  public synchronized boolean getStatus() {
    return PRINT;
  }

  public synchronized void printlnAlways(String output) {
    String callerName = Thread.currentThread().getStackTrace()[2].getMethodName();
    String time = ZonedDateTime.now().format(DateTimeFormatter.ISO_LOCAL_TIME);
    System.out.println(here() + " [" + time + "] (in " + callerName + ") : " + output);
  }
}
