package nfjglbsw.examples.fib;

import java.io.Serializable;
import nfjglbsw.multiworker.NFJTask;

public class FibTask extends NFJTask<Long> implements Serializable {
  int continuation = 0;
  final int n;
  public long childID1, childID2; // necessary for join
  private volatile long childResult1;
  private volatile long childResult2;

  public FibTask(int n, NFJTask<Long> parentTask, long taskID) {
    super(parentTask, taskID);
    this.n = n;
  }

  public FibTask(FibTask source) { // copy constructor
    super(source);
    this.continuation = source.continuation;
    this.n = source.n;
    this.childID1 = source.childID1;
    this.childID2 = source.childID2;
    this.childResult1 = source.childResult1;
    this.childResult2 = source.childResult2;
  }

  @Override
  public boolean reachedJoin() {
    return continuation >= 3;
  }

  @Override
  public void receiveChildResult(long childID, Long val) {
    if (childID == childID1) {
      childResult1 = val;
    } else if (childID == childID2) {
      childResult2 = val;
    } else {
      System.out.println(
          "receiveChildResult with invalid childID="
              + childID
              + " (childID1="
              + childID1
              + ", childID2="
              + childID2
              + ")");
    }
  }

  @Override
  public Long getChildResult(long childID) {
    if (childID == childID1) {
      return childResult1;
    } else if (childID == childID2) {
      return childResult2;
    } else {
      System.out.println(
          "getChildResult with invalid childID="
              + childID
              + " (childID1="
              + childID1
              + ", childID2="
              + childID2
              + ")");
      return -1L;
    }
  }

  @Override
  public void mergeResults(NFJTask<Long> other) {
    FibTask otherFib = (FibTask) other;

    this.childResult1 = otherFib.childResult1;
    this.childResult2 = otherFib.childResult2;
    this.childID1 = otherFib.childID1;
    this.childID2 = otherFib.childID2;
    this.continuation = otherFib.continuation;
    setStolen(false);
  }

}
