package nfjglbsw.examples.fib;

import static apgas.Constructs.*;

import apgas.Configuration;
import nfjglbsw.examples.util.ExampleHelper;
import nfjglbsw.examples.util.LongSum;
import nfjglbsw.multiworker.GLBFactory;
import nfjglbsw.multiworker.GLBMultiWorkerConfiguration;
import nfjglbsw.multiworker.GLBcomputer;
import org.apache.commons.cli.*;

public class StartFib {

  static final int N_DEFAULT = 12;
  static final int CUTOFF_DEFAULT = 2;

  static int n = N_DEFAULT;
  static int cutOff = CUTOFF_DEFAULT;

  private static void parseArguments(String[] args, boolean verbose) {
    Options options = new Options();

    options.addOption("n", true, "n'th fibonacci number");
    options.addOption("cutOff", true, "sequential cut off");

    CommandLineParser parser = new DefaultParser();
    CommandLine cmd = null;
    try {
      cmd = parser.parse(options, args);
    } catch (ParseException e) {
      e.printStackTrace();
    }

    n = Integer.parseInt(cmd.getOptionValue("n", String.valueOf(N_DEFAULT)));
    cutOff = Integer.parseInt(cmd.getOptionValue("cutOff", String.valueOf(CUTOFF_DEFAULT)));

    if (verbose) {
      System.out.println("FIB config:\n" + "  n=" + n + "\n" + "  cutOff=" + cutOff + "\n");
    }
  }

  public static void main(String[] args) {
    ExampleHelper.printStartMessage(StartFib.class.getName());
    ExampleHelper.configureAPGAS(false);
    Configuration.printAllConfigs();
    GLBMultiWorkerConfiguration.printAllConfigs();
    parseArguments(args, true);
    //    ExampleHelper.printAllFJsScheduled(5);

    final int repetitions = GLBMultiWorkerConfiguration.GLB_MULTIWORKER_BENCHMARKREPETITIONS.get();

    for (int i = 0; i < repetitions; i++) {
      GLBcomputer<LongSum, Fib> glb = new GLBFactory<LongSum, Fib>().setupGLB(places());

      final int _cutOff = cutOff;

      final Fib fib = new Fib(_cutOff);
      fib.init(n, glb);

      LongSum result = glb.computeDynamic(fib, () -> new LongSum(0L), () -> new Fib(_cutOff));

      System.out.println("Run " + (i + 1) + "/" + repetitions + "; " + result.sum + "; ");
      // + glb.getLog().computationTime / 1e9
      // + "; ");

      // System.out.println("Process time: " + glb.getLog().computationTime / 1e9 + " seconds");

      // glb.getLog().printShort(System.out);
      // glb.getLog().printAll(System.out);
      System.out.println();
      System.out.println("#############################################################");

      System.out.println("#############################################################");
      System.out.println();
    }
  }
}
