package nfjglbsw.examples.nqueens;

import nfjglbsw.examples.util.LongSum;
import nfjglbsw.multiworker.Bag;
import nfjglbsw.multiworker.GLBWorker;
import nfjglbsw.multiworker.NFJTask;

import java.io.Serializable;
import java.util.Iterator;

public class NQueens implements Bag<NQueens, LongSum>, Serializable {

  public final int QUEENS;
  public final int cutOff;
  public NQueensTaskQueue queue;
  int workerID = -1;

  public NQueens(final int queens, final int cutOff, final int initSize) {
    this.cutOff = cutOff;
    this.QUEENS = queens;
    this.queue = new NQueensTaskQueue(initSize);
  }

  public static int[] extendRight(final int[] src, final int newValue) {
    final int[] res = new int[src.length + 1];
    System.arraycopy(src, 0, res, 0, src.length);
    res[src.length] = newValue;
    return res;
  }

  /*
   * <a> contains array of <n> queen positions.  Returns 1
   * if none of the queens conflict, and returns 0 otherwise.
   */
  public static boolean isBoardValid(final int n, final int[] a) {
    int i, j;
    int p, q;
    for (i = 0; i < n; i++) {
      p = a[i];
      for (j = (i + 1); j < n; j++) {
        q = a[j];
        if (q == p || q == p - (j - i) || q == p + (j - i)) {
          return false;
        }
      }
    }
    return true;
  }

  @Override
  public boolean isEmpty() {
    return queue.isEmpty();
  }

  @Override
  public boolean isSplittable() {
    for (NQueensTask task : queue) {
      if (!task.reachedJoin()) {
        return this.queue.size() > 1;
      }
    }
    return false;
  }

  @Override
  public void merge(NQueens other) {
    this.queue.pushArrayFirst(other.queue.toArray());
  }

  @Override
  public void merge(NFJTask task) {
    this.queue.addFirst((NQueensTask) task);
  }

  @Override
  public NFJTask[] asTasks() {
    NFJTask[] tasks = new NFJTask[this.queue.size()];
    int i = 0;
    for (NFJTask task : this.queue) {
      tasks[i++] = task;
    }
    return tasks;
  }

  @Override
  public NFJTask process(int workAmount, LongSum sharedObject, GLBWorker<LongSum, NQueens> computer) {
    workerID = computer.workerID;
    NQueensTask currentTask = queue.removeFirst();
    try {
      int j = 0;
      while (j < workAmount) {
//      try { Thread.sleep(50); } catch (Exception e) {}
        ++j;
        final int[] currentA = currentTask.b;
        final int currentD = currentTask.depth;

        int i = currentTask.continuation;
        if (i < QUEENS) {
          final int ii = i;

          //System.out.println(computer.workerID + " processing task depth = " + currentD + " i = " + i + " atJoin = " + currentTask.atJoin + " cont = " + currentTask.continuation + " task.ID = " + currentTask.getID());

          ++currentTask.continuation;
          if (currentTask.continuation >= QUEENS) {
            currentTask.atJoin = true;
          }
          if (currentD < cutOff) {
            final int[] b = extendRight(currentA, ii);
            if (isBoardValid((currentD + 1), b)) {
              NQueensTask childTask = new NQueensTask(b, currentD + 1, currentTask, computer.generateNewTaskID());
              //System.out.println("task.ID = " + currentTask.getID() + " spawning child.id = " + childTask.getID());
              queue.addFirst(currentTask);
              currentTask.childResults.add(0L);
              currentTask.childIDMap.put(childTask.getID(), currentTask.childIDMap.size());
              currentTask = childTask;
              continue;
            }
          } else {
            Long result = 0L;
            for (i = 0; i < QUEENS; ++i) {
              final int[] b = extendRight(currentA, i);
              if (isBoardValid((currentD + 1), b)) {
                final int[] b2 = new int[QUEENS];
                try {
                  System.arraycopy(b, 0, b2, 0, b.length);

                } catch (Throwable t) {
                  t.printStackTrace();
                }
                result += nqueensKernelSeq(b2, currentD + 1);
              }
            }
            currentTask.childResults.add(result);
            currentTask.continuation = QUEENS;
            currentTask.atJoin = true;
            computer.join(currentTask, new long[0]);
            //System.out.println("cutOff task.ID = " + currentTask.getID() + " result = " + result);
            currentTask = null;
          }
        } else if (currentTask.continuation == QUEENS) { // join
          currentTask.atJoin = true;
          //System.out.println("joining task.id = " + currentTask.getID() + " with numChilds = " + currentTask.childIDMap.size());
          long childIDs[] = new long[currentTask.childIDMap.size()];
          int idx = 0;
          for (Long childID : currentTask.childIDMap.keySet()) {
            if (currentTask.getID() == 0) {
              //System.out.println("root task has child ids = " + childID);
            }
            childIDs[idx++] = childID;
          }
          ++currentTask.continuation;
          computer.join(currentTask, childIDs);
          currentTask = null;
        } else { // return
          //System.out.println("returning task.id = " + currentTask.getID() + " at cont = " + currentTask.continuation);
          long result = 0;
          for (Long childResult : currentTask.childResults) {
            //System.out.println("task.id = " + currentTask.getID() + " adding child result = " + childResult);
            result += childResult;
          }
          if (currentTask.getParent() == null) {
            System.out.println("RESULT IS " + result + " currentTask.id = " + currentTask.getID());
          }

          computer.parentReturn(currentTask, result);
          currentTask = null;
        }
        if (this.queue.size() > 0 && j < workAmount) {
          currentTask = this.queue.removeFirst();
        } else {
          break;
        }
      }
    } catch (Exception e) { e.printStackTrace(); }
    return currentTask;
  }

  public Long nqueensKernelSeq(final int[] a, final int depth) {
    if (QUEENS == depth) {
      return 1L;
    }

    Long result = 0L;
    for (int i = 0; i < QUEENS; i++) {
      a[depth] = i;
      if (isBoardValid((depth + 1), a)) {
        result += nqueensKernelSeq(a, depth + 1);
      }
    }
    return result;
  }

  @Override
  public NQueens split(boolean takeAll, int numTasks) {
    NQueens other = new NQueens(QUEENS, cutOff,1);

    if (!isSplittable() && takeAll) {
      other.queue.pushArrayFirst(this.queue.getFromLast(this.queue.size()));
      return other;
    }

    int joined = 0;
    for (Iterator<NQueensTask> iterator = queue.descendingIterator(); iterator.hasNext(); ) {
      NQueensTask task = iterator.next();
      if (!task.reachedJoin()) {
        NQueensTask[] fromLast = this.queue.getFromLast(joined);
        other.queue.addLast(this.queue.removeLast());
        this.queue.pushArrayLast(fromLast);
        break;
      } else {
        joined++;
      }
    }
    return other;
  }

  @Override
  public long getCurrentTaskCount() {
    return queue.size();
  }

  @Override
  public void initStaticTasks(int workerId) {
    // Never called because computeDynamic is used
  }

  @Override
  public NQueens deepCopy() {
    NQueens other = new NQueens(QUEENS, cutOff, this.queue.size());
    for (NQueensTask task : this.queue) {
      other.queue.addLast(new NQueensTask(task));
    }
    return other;
  }

  public void init() {
    this.queue.addFirst(new NQueensTask(new int[0], 0, null, 0));
  }
}
