package nfjglbsw.examples.syntheticBenchmark;

import static apgas.Constructs.places;

import apgas.Configuration;
import nfjglbsw.examples.util.ExampleHelper;
import nfjglbsw.examples.util.LongSum;
import nfjglbsw.multiworker.GLBFactory;
import nfjglbsw.multiworker.GLBMultiWorkerConfiguration;
import nfjglbsw.multiworker.GLBcomputer;
import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.CommandLineParser;
import org.apache.commons.cli.DefaultParser;
import org.apache.commons.cli.Options;
import org.apache.commons.cli.ParseException;

public class StartSynthetic {

  static final long BALLAST_DEFAULT = 0;
  static final long TASKSPERWORKER_DEFAULT = 1_000_000;
  static final long TOTALDURATION_DEFAULT = 20_000;
  static final long TASK_DURATION_VARIANCE_DEFAULT = 0;

  static long ballast = BALLAST_DEFAULT;
  static long tasksPerWorker = TASKSPERWORKER_DEFAULT;
  static long totalDuration = TOTALDURATION_DEFAULT;
  static long taskDurationVariance = TASK_DURATION_VARIANCE_DEFAULT;

  private static void parseArguments(String[] args, boolean verbose) {
    Options options = new Options();

    options.addOption("b", true, "Task ballast in bytes (Default 0 KiB)");
    options.addOption(
        "t", true, "Task count per worker (Default 1_000_000)");
    options.addOption("g", true, "Total duration in milliseconds (Default 10_000)");
    options.addOption("u", true, "Task duration variance percentage (Default 0");

    CommandLineParser parser = new DefaultParser();
    CommandLine cmd = null;
    try {
      cmd = parser.parse(options, args);
    } catch (ParseException e) {
      e.printStackTrace();
    }

    ballast = Long.parseLong(cmd.getOptionValue("b", String.valueOf(BALLAST_DEFAULT)));
    tasksPerWorker =
        Long.parseLong(cmd.getOptionValue("t", String.valueOf(TASKSPERWORKER_DEFAULT)));
    totalDuration = Long.parseLong(cmd.getOptionValue("g", String.valueOf(TOTALDURATION_DEFAULT)));
    taskDurationVariance =
        Long.parseLong(cmd.getOptionValue("u", String.valueOf(TASK_DURATION_VARIANCE_DEFAULT)));

    if (verbose) {
      System.out.println(
          "Synthetic config:\n"
              + "  ballast="
              + ballast
              + "\n"
              + "  tasksPerWorker="
              + tasksPerWorker
              + "\n"
              + "  totalDuration(in milliseconds)="
              + totalDuration
              + "\n"
              + "  totalDuration(in seconds)="
              + totalDuration / 1e3
              + "\n"
              + "  taskDurationVariance="
              + taskDurationVariance
              + "\n");
    }
  }

  public static void main(String[] args) {
    ExampleHelper.printStartMessage(StartSynthetic.class.getName());
    ExampleHelper.configureAPGAS(false);
    Configuration.printAllConfigs();
    GLBMultiWorkerConfiguration.printAllConfigs();
    parseArguments(args, true);
    //    ExampleHelper.printAllFJsScheduled(5);

    final int repetitions = GLBMultiWorkerConfiguration.GLB_MULTIWORKER_BENCHMARKREPETITIONS.get();

    final long _ballast = ballast;
    final long _tasksPerWorker = tasksPerWorker;
    final long _totalDuration = totalDuration;
    final long _taskDurationVariance = taskDurationVariance;

    final int workerPerPlace = GLBMultiWorkerConfiguration.GLB_MULTIWORKER_WORKERPERPLACE.get();
    final int wholeNumPlaces = places().size();
    long expectedResult = 0;

    for (int i = 0; i < repetitions; i++) {

      final LongSum sum;
      final GLBcomputer<LongSum, SyntheticQueue> glb;

      // maxChildren=0, wird gleich durch initDynamic gesetzt
      final SyntheticQueue syntheticQueue = new SyntheticQueue(_taskDurationVariance, 0);
      expectedResult = syntheticQueue.initDynamic(_tasksPerWorker, _totalDuration, _ballast);
      final long _maxChildren = syntheticQueue.maxChildren;
      //        final float _variance = syntheticQueue.variance;
      //        final long _depth = syntheticQueue.depth;

      glb = new GLBFactory<LongSum, SyntheticQueue>().setupGLB(places());

      sum =
          glb.computeDynamic(
              syntheticQueue,
              () -> new LongSum(0),
              () -> new SyntheticQueue(_taskDurationVariance, _maxChildren));
    }
  }
}
