package nfjglbsw.examples.syntheticBenchmark;

import nfjglbsw.examples.util.LongSum;
import nfjglbsw.multiworker.Bag;
import nfjglbsw.multiworker.GLBWorker;
import nfjglbsw.multiworker.NFJTask;

import java.util.Iterator;

public class SyntheticQueue extends Synthetic implements Bag<SyntheticQueue, LongSum> {

  public SyntheticQueue(final long durationVariance, final long maxChildren) {
    super(durationVariance, maxChildren);
  }

  public SyntheticQueue(final long durationVariance, final long maxChildren, final int initialCapacity) {
    super(durationVariance, maxChildren, initialCapacity);
  }

  @Override
  public boolean isEmpty() {
    return this.tasks.size() == 0;
  }

  @Override
  public boolean isSplittable() {
    for (SyntheticTask task : tasks) {
      if (!task.reachedJoin() && task.depth > 0) {
        return this.tasks.size() > 1;
      }
    }
    return false;
  }

  @Override
  public void merge(SyntheticQueue syntheticQueue) {
    this.tasks.pushArrayFirst(syntheticQueue.tasks.toArray());
    this.diff += syntheticQueue.diff;
    this.result += syntheticQueue.result;
  }

  @Override
  public void merge(NFJTask other) {
    this.tasks.addFirst((SyntheticTask) other);
  }

  @Override
  public NFJTask[] asTasks() {
    NFJTask[] tasks = new NFJTask[this.tasks.size()];
    int i = 0;
    for (NFJTask task : this.tasks) {
      tasks[i++] = task;
    }
    return tasks;
  }

  @Override
  public NFJTask process(int workAmount, LongSum sharedObject, GLBWorker<LongSum, SyntheticQueue> computer) {
    if (this.tasks.size() == 0) return null;
    int i = 0;
    SyntheticTask task = this.tasks.removeFirst();
    while (i < workAmount) {
      ++i;
      task = calculate(task, computer);
      if (task == null) {
        if (this.tasks.size() > 0 && i < workAmount) {
          task = this.tasks.removeFirst();
        } else {
          break;
        }
      }
    }
    return task;
  }

  public SyntheticQueue split(boolean takeAll, int numTasks) {
    SyntheticQueue other = new SyntheticQueue(durationVariance, maxChildren, 1);

    if (!isSplittable() && takeAll) {
      other.tasks.pushArrayFirst(this.tasks.getFromLast(this.tasks.size()));
      other.diff = diff;
      diff = 0;
      return other;
    }

    int joined = 0;
    for (Iterator<SyntheticTask> iterator = tasks.descendingIterator(); iterator.hasNext(); ) {
      SyntheticTask task = iterator.next();
      if (!task.reachedJoin() && task.depth > 0) {
        SyntheticTask[] fromLast = this.tasks.getFromLast(joined);
        other.tasks.addLast(this.tasks.removeLast());
        this.tasks.pushArrayLast(fromLast);
        break;
      } else {
        joined++;
      }
    }
    return other;
  }

  @Override
  public long getCurrentTaskCount() {
    return this.tasks.size();
  }

  @Override
  public void initStaticTasks(int workerId) {
    // unused
  }

  @Override
  public SyntheticQueue deepCopy() {
    SyntheticQueue other = new SyntheticQueue(this.durationVariance, this.maxChildren, 1);
    for (SyntheticTask task : this.tasks) {
      other.tasks.addLast(new SyntheticTask(task));
    }
    return other;
  }
}
