package nfjglbsw.examples.syntheticBenchmark;

import nfjglbsw.multiworker.NFJTask;

import java.io.Serializable;
import java.util.HashMap;
import java.util.Map;

public class SyntheticTask extends NFJTask<Long> implements Serializable {

  private static final long serialVersionUID = 2282792464012580417L;

  byte[] ballast;
  long seed;
  long depth;
  long duration;
  int continuation;
  long processedTime;
  long childResults[];
  Map<Long, Integer> childIDMap;

  public SyntheticTask(long ballastInBytes, long duration, int maxChildren, NFJTask<Long> parentTask, long taskID) {
    super(parentTask, taskID);
    this.ballast = new byte[(int) ballastInBytes];
    this.duration = duration;
    this.childResults = new long[maxChildren];
    this.childIDMap = new HashMap<>();
  }

  public SyntheticTask(SyntheticTask source) { // copy constructor
    super(source);
    this.seed = source.seed;
    this.depth = source.depth;
    this.duration = source.duration;
    this.continuation = source.continuation;
    this.childResults = source.childResults;
    this.childIDMap = source.childIDMap;
    this.ballast = source.ballast;
    this.processedTime = source.processedTime;
  }

  public SyntheticTask(long ballastInBytes, long seed, long depth, long duration, int maxChildren, NFJTask<Long> parentTask, long taskID) {
    this(ballastInBytes, duration, maxChildren, parentTask, taskID);
    this.seed = seed;
    this.depth = depth;
  }

  @Override
  public boolean reachedJoin() {
    return continuation >= childResults.length;
  }

  @Override
  public void receiveChildResult(long childID, Long val) {
    childResults[childIDMap.get(childID)] = val;
  }

  @Override
  public Long getChildResult(long childID) {
    return childResults[childIDMap.get(childID)];
  }

  @Override
  public void mergeResults(NFJTask<Long> other) {
    SyntheticTask otherTask = (SyntheticTask) other;

    this.continuation = otherTask.continuation;
    this.childResults = otherTask.childResults;
    this.childIDMap = otherTask.childIDMap;
    this.processedTime = otherTask.processedTime;
    setStolen(false);
  }
}
