package nfjglbsw.examples.uts;

import static apgas.Constructs.*;

import apgas.Configuration;
import nfjglbsw.examples.util.ExampleHelper;
import nfjglbsw.examples.util.LongSum;
import nfjglbsw.multiworker.GLBFactory;
import nfjglbsw.multiworker.GLBMultiWorkerConfiguration;
import nfjglbsw.multiworker.GLBcomputer;
import org.apache.commons.cli.*;

public class StartUTS {

  static final int BRANCHING_DEFAULT = 4;
  static final int SEED_DEFAULT = 19;
  static final int DEPTH_DEFAULT = 13;
  static final int QSIZE_DEFAULT = 64;
  static final int CUTOFF_DEFAULT = 0;

  static int branching = BRANCHING_DEFAULT;
  static int seed = SEED_DEFAULT;
  static int depth = DEPTH_DEFAULT;
  static int qSize = QSIZE_DEFAULT;
  static int cutOff = CUTOFF_DEFAULT;

  private static void parseArguments(String[] args, boolean verbose) {
    Options options = new Options();
    options.addOption("b", true, "Branching factor");
    options.addOption("s", true, "Seed (0 <= r < 2^31)");
    options.addOption("d", true, "Tree depth");
    options.addOption("cutOff", true, "Cutoff");

    CommandLineParser parser = new DefaultParser();
    CommandLine cmd = null;
    try {
      cmd = parser.parse(options, args);
    } catch (ParseException e) {
      e.printStackTrace();
    }

    branching = Integer.parseInt(cmd.getOptionValue("b", String.valueOf(BRANCHING_DEFAULT)));
    seed = Integer.parseInt(cmd.getOptionValue("s", String.valueOf(SEED_DEFAULT)));
    depth = Integer.parseInt(cmd.getOptionValue("d", String.valueOf(DEPTH_DEFAULT)));
    cutOff = Integer.parseInt(cmd.getOptionValue("cutOff", String.valueOf(CUTOFF_DEFAULT)));

    if (verbose) {
      System.out.println(
          "UTS config:\n"
              + "  branching="
              + branching
              + "\n"
              + "  seed="
              + seed
              + "\n"
              + "  depth="
              + depth
              + "\n"
              + "  cutOff="
              + cutOff
              + "\n");
    }
  }

  public static void main(String[] args) {
    ExampleHelper.printStartMessage(StartUTS.class.getName());
    ExampleHelper.configureAPGAS(false);
    Configuration.printAllConfigs();
    GLBMultiWorkerConfiguration.printAllConfigs();
    parseArguments(args, true);
    //    ExampleHelper.printAllFJsScheduled(5);

    final int repetitions = GLBMultiWorkerConfiguration.GLB_MULTIWORKER_BENCHMARKREPETITIONS.get();

    for (int i = 0; i < repetitions; i++) {
      GLBcomputer<LongSum, UTS> glb = new GLBFactory<LongSum, UTS>().setupGLB(places());

      final long _cutOff = cutOff;

      final UTS uts = new UTS(_cutOff);
      uts.init(seed, depth, glb);

      LongSum result = glb.computeDynamic(uts, () -> new LongSum(0L), () -> new UTS(_cutOff));

      System.out.println(
          "Run "
              + (i + 1)
              + "/"
              + repetitions
              + "; "
              + result.sum
              //              + "; "
              //              + glb.getLog().computationTime / 1e9
              + "; ");

      //      System.out.println("Process time: " + glb.getLog().computationTime / 1e9 + "
      // seconds");

      //      glb.getLog().printShort(System.out);
      //      glb.getLog().printAll(System.out);
      System.out.println();
      System.out.println("#############################################################");

      System.out.println("#############################################################");
      System.out.println();
    }
  }
}
