package nfjglbsw.multiworker;

import java.io.Serializable;
import java.util.ArrayList;

public abstract class NFJTask<T> implements Serializable {
  final long id;
  transient volatile NFJTask<T> parentTask;
  public volatile ArrayList<Integer> victims;
  transient volatile boolean stolen;

  public abstract boolean reachedJoin();

  public abstract void receiveChildResult(long childID, T val);

  public abstract T getChildResult(long childID);

  public NFJTask(NFJTask<T> parentTask, long taskID) {
    this.id = taskID;
    this.parentTask = parentTask;
    this.victims = new ArrayList<>();
    this.stolen = false;
  }

  public NFJTask(NFJTask source) { // copy constructor
    this.id = source.id;
    this.victims = (ArrayList<Integer>) source.victims.clone();
    // dont copy transient fields
  }

  public boolean isStolen() {
    return stolen;
  }

  public void setStolen(boolean stolen) {
    this.stolen = stolen;
  }

  public long getID() {
    return this.id;
  }

  public NFJTask<T> getParent() {
    return this.parentTask;
  }

  public abstract void mergeResults(NFJTask<T> other);
}
