package nfjglbswft.examples.uts;

import nfjglbswft.multiworker.NFJTask;

import java.io.Serializable;
import java.util.HashMap;

public class UTSTask extends NFJTask<Long> implements Serializable {
  int continuation = 0;
  int numChildren = 0;
  public long childResults[];
  HashMap<Long, Integer> childIDMap;

  protected SHA1Rand utsParent;
  protected SHA1Rand sha1Rand;
  protected int utsID;

  @Override
  public boolean reachedJoin() {
    return continuation >= numChildren;
  }

  public UTSTask(UTSTask source) { // copy constructor
    super(source);
    this.continuation = source.continuation;
    this.numChildren = source.numChildren;
    this.childResults = source.childResults;
    this.childIDMap = source.childIDMap;
    this.utsParent = source.utsParent;
    this.sha1Rand = source.sha1Rand;
    this.utsID = source.utsID;
  }

  public UTSTask(SHA1Rand utsParent, int utsID, UTSTask parentTask, long taskID) {
    super(parentTask, taskID);
    this.utsParent = utsParent;
    this.utsID = utsID;
    this.childIDMap = new HashMap<>();
  }

  @Override
  public void receiveChildResult(long childID, Long val) {
    childResults[childIDMap.get(childID)] = val;
  }

  @Override
  public Long getChildResult(long childID) {
    return childResults[childIDMap.get(childID)];
  }

  @Override
  public void mergeResults(NFJTask<Long> other) {
    UTSTask otherUTS = (UTSTask) other;

    this.childResults = otherUTS.childResults;
    this.childIDMap = otherUTS.childIDMap;
    this.continuation = otherUTS.continuation;
    setStolen(false);

  }

  public SHA1Rand getUTSParent() {
    return utsParent;
  }

  public int getUTSID() {
    return utsID;
  }
}
